﻿using System.Collections.Generic;
using UnityEngine;

public class Ant : AntObject
{
	const float WANDER_INTERVAL = 1.5f, WANDER_INTERVAL_RANDOM = 0.5f;
	const float WANDER_MIN_DISTANCE = 2.0f, WANDER_MAX_DISTANCE = 4.0f;
	const float SCAN_INTERVAL = 0.1f;
	const float THOUGHT_TRANSLATE_INTERVAL = 0.1f;
	const float THOUGHT_TRANSLATE_DELAY = 0.5f, THOUGHT_TRANSLATE_DELAY_RANDOM = 0.25f;

	public Ant(Anthill anthill, GameObject antSprite, Vector3 position, float scanRadius, float translateRadius)
	  : base(anthill, antSprite, position)
	{
		this.scanRadius = scanRadius;
		this.translateRadius = translateRadius;
		pathFinder = (PathFinder) sprite.GetComponent <PathFinder> ();
		cloud = (CloudThought) sprite.GetComponentInChildren <CloudThought> ();
	}

	override public void setThought(Thought thought)
	{
		base.setThought(thought);
		if (thought != null)
		{
			cloud.setSprite(thought.getCloudType());
			thoughtTranslateDelay = THOUGHT_TRANSLATE_DELAY + Random.value * THOUGHT_TRANSLATE_DELAY_RANDOM * 2 - THOUGHT_TRANSLATE_DELAY_RANDOM;
		}
		else
		{
			cloud.setSprite ((int) Cloud.Quest);
		}
	}

	public void update(float dt)
	{
		if (getSprite() != null)
		{
			position = sprite.transform.position;
		}

		pathFinder.setResource (resource);

		scan(dt);
		if (thought == null)
		{
			if (anthill.isAntTooFarAway(this) || resource != Resource.None)
			{
				setThought(new HomeThought(anthill.makeHomeObject(sprite.transform.position), null));
			}
			else
			{
				wander(dt);
			}
		}
		else
		{
			if (thought.isTranslateable())
			{
				translateThought(dt);
			}
			thought.think(this, dt);
		}

	}

	public void wander(float dt)
	{
		if (wanderInterval == 0)
		{
			wanderInterval = WANDER_INTERVAL + (Random.value * WANDER_INTERVAL_RANDOM * 2 - WANDER_INTERVAL_RANDOM);
		}

		wanderTime += dt;
		if (wanderTime >= wanderInterval)
		{
			Vector3 myPos = getSprite().transform.position;
			Vector3 wanderPos = myPos + Random.insideUnitSphere.normalized * WANDER_MIN_DISTANCE
			                  * (1 + Random.value * (WANDER_MAX_DISTANCE - WANDER_MIN_DISTANCE));
			wanderPos.y = 0;
			move(wanderPos);
			wanderTime %= wanderInterval;
			wanderTime = wanderInterval = 0;
		}
	}

	public void move(Vector3 targetPos)
	{
		pathFinder.setGoal(targetPos);
	}

	public void sacrifice(EnemyAnt target)
	{
		target.takeDamage ();
		die ();
	}

	public void scan(float dt)
	{
		scanTime += dt;
		if (scanTime < SCAN_INTERVAL) return;

		List <AntObject> objects = anthill.getObjects();
		if (objects.Count == 0) return;
		foreach (AntObject obj in objects)
		{
			if (obj == this) continue;

			Vector3 myPos = getSprite().transform.position;
			Vector3 objPos = (obj.getSprite() != null ? obj.getSprite().transform.position : obj.getPosition());
			if ((myPos - objPos).magnitude <= scanRadius)
			{
				// hardcoding - ignoring resources if we're going home
				if (thought is HomeThought && obj is FarmPoint) continue;

				if (obj.getThought() != null && (thought == null || thought.getRating() < obj.getThought().getRating()))
				{
					setThought(obj.getThought().clone());
				}
			}
		}

		scanTime %= SCAN_INTERVAL;
	}

	public void translateThought(float dt)
	{
		thoughtTranslateDelay -= dt;
		if (thoughtTranslateDelay > 0) return;
		thoughtTranslateTime += dt;
		if (thoughtTranslateTime < THOUGHT_TRANSLATE_INTERVAL) return;

		List <Ant> ants = anthill.getAnts();
		foreach (Ant ant in ants)
		{
			if (ant == this) continue;

			Vector3 myPos = getSprite().transform.position;
			Vector3 otherAntPos = ant.getSprite().transform.position;
			if ((myPos - otherAntPos).magnitude <= translateRadius)
			{
				if (ant.getThought() == null || thought.shouldTranslate(ant, ant.getThought().getRating()))
				{
					ant.setThought(thought.clone());
				}
			}
		}

		thoughtTranslateTime %= THOUGHT_TRANSLATE_INTERVAL;
	}

	public void die()
	{
		anthill.removeAnt(this);
		GameObject.Destroy(sprite);
	}

	public bool hasReachedTarget()
	{
		return pathFinder.hasReachedTarget();
	}

	public Resource getResource()
	{
		return resource;
	}

	public void setResource(Resource resource)
	{
		this.resource = resource;
	}
	public CloudThought getCloud()
	{
		return cloud;
	}

	protected Resource resource;
	protected float scanRadius, translateRadius;
	protected PathFinder pathFinder;
	protected CloudThought cloud;
	float wanderInterval = 0, wanderTime = 0;
	float scanTime = 0;
	float thoughtTranslateTime = 0, thoughtTranslateDelay = 0;
}
